@@ -2,6 +2,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "t/test-full.pb-c.h"
+#include "t/test-optimized.pb-c.h"
 #include "t/generated-code2/test-full-cxx-output.inc"
 
 #define TEST_ENUM_SMALL_TYPE_NAME   Foo__TestEnumSmall
@@ -95,6 +96,8 @@ binary_data_equals (ProtobufCBinaryData a, ProtobufCBinaryData b)
 {
   if (a.len != b.len)
     return 0;
+  if (a.len == 0 && b.len == 0)
+    return 1;
   return memcmp (a.data, b.data, a.len) == 0;
 }
 
@@ -129,6 +132,7 @@ static void test_enum_small (void)
 
   assert (sizeof (Foo__TestEnumSmall) == 4);
 
+  DO_TEST(NEG_VALUE);
   DO_TEST(VALUE);
   DO_TEST(OTHER_VALUE);
 
@@ -156,6 +160,8 @@ static void test_enum_big (void)
     free (data); \
   }while(0)
 
+  DO_ONE_TEST(VALUENEG123456, -123456, 10);
+  DO_ONE_TEST(VALUENEG1, -1, 10);
   DO_ONE_TEST(VALUE0, 0, 1);
   DO_ONE_TEST(VALUE127, 127, 1);
   DO_ONE_TEST(VALUE128, 128, 2);
@@ -307,7 +313,9 @@ static void test_required_uint64 (void)
   DO_TEST(THOUSAND, test_required_uint64_thou);
   DO_TEST(MILLION, test_required_uint64_mill);
   DO_TEST(BILLION, test_required_uint64_bill);
+  DO_TEST(BILLION*20, test_required_uint64_20_bill);
   DO_TEST(TRILLION, test_required_uint64_tril);
+  DO_TEST(TRILLION*20, test_required_uint64_20_tril);
   DO_TEST(QUADRILLION, test_required_uint64_quad);
   DO_TEST(QUINTILLION, test_required_uint64_quint);
   DO_TEST(UINT64_MAX, test_required_uint64_max);
@@ -372,6 +380,7 @@ static void test_required_TestEnum (void)
 #define DO_TEST(value, example_packed_data) \
   DO_TEST_REQUIRED(Enum, ENUM, enum, value, example_packed_data, NUMERIC_EQUALS)
 
+  DO_TEST (FOO__TEST_ENUM__VALUENEG1, test_required_enum_neg1);
   DO_TEST (FOO__TEST_ENUM__VALUE0, test_required_enum_0);
   DO_TEST (FOO__TEST_ENUM__VALUE1, test_required_enum_1);
   DO_TEST (FOO__TEST_ENUM__VALUE127, test_required_enum_127);
@@ -627,6 +636,7 @@ static void test_optional_TestEnum (void)
 #define DO_TEST(value, example_packed_data) \
   DO_TEST_OPTIONAL(test_enum, value, example_packed_data, NUMERIC_EQUALS)
 
+  DO_TEST (FOO__TEST_ENUM__VALUENEG1, test_optional_enum_neg1);
   DO_TEST (FOO__TEST_ENUM__VALUE0, test_optional_enum_0);
   DO_TEST (FOO__TEST_ENUM__VALUE1, test_optional_enum_1);
   DO_TEST (FOO__TEST_ENUM__VALUE127, test_optional_enum_127);
@@ -908,6 +918,7 @@ static void test_oneof_TestEnum (void)
 #define DO_TEST(value, example_packed_data) \
   DO_TEST_ONEOF(test_enum, TEST_ENUM, value, example_packed_data, GENERIC_ASSIGN, NUMERIC_EQUALS)
 
+  DO_TEST (FOO__TEST_ENUM__VALUENEG1, test_optional_enum_neg1);
   DO_TEST (FOO__TEST_ENUM__VALUE0, test_optional_enum_0);
   DO_TEST (FOO__TEST_ENUM__VALUE1, test_optional_enum_1);
   DO_TEST (FOO__TEST_ENUM__VALUE127, test_optional_enum_127);
@@ -1565,9 +1576,10 @@ test_enum_descriptor (const ProtobufCEnumDescriptor *desc)
       const ProtobufCEnumValue *vv;
       const ProtobufCEnumValue *vn;
       vv = protobuf_c_enum_descriptor_get_value (desc, sv->value);
-      vn = protobuf_c_enum_descriptor_get_value_by_name (desc, sv->name);
       assert (sv == vv);
-      assert (sv == vn);
+      vn = protobuf_c_enum_descriptor_get_value_by_name (desc, sv->name);
+      if (sv->name != NULL)
+         assert (sv == vn);
     }
   for (i = 0; i < desc->n_value_names; i++)
     {
@@ -1588,12 +1600,31 @@ test_enum_by_name (const ProtobufCEnumDescriptor *desc,
   assert (v->value == expected_value);
 }
 
+static void
+test_lite_enum (void)
+{
+  const ProtobufCEnumDescriptor *desc = &foo__test_enum_lite__descriptor;
+  const ProtobufCEnumValue *v;
+
+  v = protobuf_c_enum_descriptor_get_value_by_name (desc, "BOO");
+  assert (v == NULL);
+
+  v = protobuf_c_enum_descriptor_get_value (desc, 0);
+  assert (v != NULL);
+  assert (v->value == 0);
+
+  v = protobuf_c_enum_descriptor_get_value (desc, 3);
+  assert (v == NULL);
+}
+
 static void
 test_enum_lookups (void)
 {
   test_enum_descriptor (&foo__test_enum__descriptor);
   test_enum_descriptor (&foo__test_enum_small__descriptor);
   test_enum_descriptor (&foo__test_enum_dup_values__descriptor);
+  test_enum_descriptor (&foo__test_enum_lite__descriptor);
+  test_lite_enum ();
 #define TEST_ENUM_DUP_VALUES(str, shortname) \
   test_enum_by_name (&foo__test_enum_dup_values__descriptor,  \
                      str, FOO__TEST_ENUM_DUP_VALUES__##shortname)
@@ -1618,9 +1649,10 @@ test_message_descriptor (const ProtobufCMessageDescriptor *desc)
       const ProtobufCFieldDescriptor *fv;
       const ProtobufCFieldDescriptor *fn;
       fv = protobuf_c_message_descriptor_get_field (desc, f->id);
-      fn = protobuf_c_message_descriptor_get_field_by_name (desc, f->name);
       assert (f == fv);
-      assert (f == fn);
+      fn = protobuf_c_message_descriptor_get_field_by_name (desc, f->name);
+      if (desc->fields_sorted_by_name != NULL)
+        assert (f == fn);
     }
 }
 static void
@@ -1629,6 +1661,7 @@ test_message_lookups (void)
   test_message_descriptor (&foo__test_mess__descriptor);
   test_message_descriptor (&foo__test_mess_optional__descriptor);
   test_message_descriptor (&foo__test_mess_required_enum__descriptor);
+  test_message_descriptor (&foo__test_mess_lite__descriptor);
 }
 
 static void
@@ -1925,6 +1958,30 @@ test_alloc_fail (void)
   free (packed);
 }
 
+static void
+test_free_unpacked_input_check_for_null_message (void)
+{
+  protobuf_c_message_free_unpacked (NULL, NULL);
+}
+
+static void
+test_free_unpacked_input_check_for_null_repeated_field (void)
+{
+  Foo__TestMess *tm = calloc(1, foo__test_mess__descriptor.sizeof_message);
+  foo__test_mess__init(tm);
+
+  tm->n_test_string = 1;
+  tm->test_string = NULL;
+
+  tm->n_test_bytes = 1;
+  tm->test_bytes = NULL;
+
+  tm->n_test_message = 1;
+  tm->test_message = NULL;
+
+  foo__test_mess__free_unpacked (tm, NULL);
+}
+
 /* This test checks that protobuf decoder is capable of detecting special
    cases of incomplete messages. The message should have at least two required
    fields field1 and field129 with positions pos1 and pos2 (no matter what the
@@ -1946,33 +2003,29 @@ static void
 test_field_flags (void)
 {
   const ProtobufCFieldDescriptor *f;
+  const ProtobufCMessageDescriptor *desc = &foo__test_field_flags__descriptor;
 
-  f = protobuf_c_message_descriptor_get_field_by_name(
-    &foo__test_field_flags__descriptor, "no_flags1");
+  f = protobuf_c_message_descriptor_get_field_by_name(desc, "no_flags1");
   assert(f != NULL);
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_PACKED));
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_DEPRECATED));
 
-  f = protobuf_c_message_descriptor_get_field_by_name(
-    &foo__test_field_flags__descriptor, "no_flags2");
+  f = protobuf_c_message_descriptor_get_field_by_name(desc, "no_flags2");
   assert(f != NULL);
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_PACKED));
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_DEPRECATED));
 
-  f = protobuf_c_message_descriptor_get_field_by_name(
-    &foo__test_field_flags__descriptor, "no_flags3");
+  f = protobuf_c_message_descriptor_get_field_by_name(desc, "no_flags3");
   assert(f != NULL);
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_PACKED));
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_DEPRECATED));
 
-  f = protobuf_c_message_descriptor_get_field_by_name(
-    &foo__test_field_flags__descriptor, "packed");
+  f = protobuf_c_message_descriptor_get_field_by_name(desc, "packed");
   assert(f != NULL);
   assert((f->flags & PROTOBUF_C_FIELD_FLAG_PACKED));
   assert(!(f->flags & PROTOBUF_C_FIELD_FLAG_DEPRECATED));
 
-  f = protobuf_c_message_descriptor_get_field_by_name(
-    &foo__test_field_flags__descriptor, "packed_deprecated");
+  f = protobuf_c_message_descriptor_get_field_by_name(desc, "packed_deprecated");
   assert(f != NULL);
   assert((f->flags & PROTOBUF_C_FIELD_FLAG_PACKED));
   assert((f->flags & PROTOBUF_C_FIELD_FLAG_DEPRECATED));
@@ -2002,7 +2055,7 @@ test_message_check(void)
   /* test with required_string not set */
   m.required_string = NULL;
   m.required_msg = &sm;
-  m.required_bytes.data = str; m.required_bytes.len = 1;
+  m.required_bytes.data = (uint8_t *)str; m.required_bytes.len = 1;
   assert(0 == protobuf_c_message_check(&m.base));
 
   /* test with required_msg not set */
@@ -2016,7 +2069,7 @@ test_message_check(void)
   assert(0 == protobuf_c_message_check(&m.base));
 
   /* test with all required fields set */
-  m.required_bytes.data = str; m.required_bytes.len = 1;
+  m.required_bytes.data = (uint8_t *)str; m.required_bytes.len = 1;
   assert(1 == protobuf_c_message_check(&m.base));
 
   /* test with incomplete required submessage */
@@ -2041,7 +2094,7 @@ test_message_check(void)
   assert(0 == protobuf_c_message_check(&m.base));
 
   /* test with correct optional bytes */
-  m.optional_bytes.data = str; m.optional_bytes.len = 1;
+  m.optional_bytes.data = (uint8_t *)str; m.optional_bytes.len = 1;
   assert(1 == protobuf_c_message_check(&m.base));
 
   /* test with repeated strings set incorrectly */
@@ -2076,7 +2129,7 @@ test_message_check(void)
   assert(0 == protobuf_c_message_check(&m.base));
 
   /* test with repeated bytes set correctly */
-  bd.data = str;
+  bd.data = (uint8_t *)str;
   assert(1 == protobuf_c_message_check(&m.base));
 
   /* test with empty repeated string vector */
@@ -2225,6 +2278,9 @@ static Test tests[] =
   { "test free unpacked", test_alloc_free_all },
   { "test alloc failure", test_alloc_fail },
 
+  { "test free unpacked input check for null message", test_free_unpacked_input_check_for_null_message },
+  { "test free unpacked input check for null repeated field", test_free_unpacked_input_check_for_null_repeated_field },
+
   { "test required_fields_bitmap", test_required_fields_bitmap },
 
   { "test field flags", test_field_flags },
